﻿Imports System.Configuration
Imports Newtonsoft.Json

Public Class AccountApiWrapper
    Private accountRequestPath As String = String.Empty
    Private accountId As String = String.Empty

    Public Sub New()
        accountRequestPath = ConfigurationManager.AppSettings("AccountPath")
        accountId = ConfigurationManager.AppSettings("AccountId")
    End Sub

    'Get Account Profile
    Public Async Function GetAccountProfile() As Task(Of String)

        'Get Authentiction code
        Dim authApi As AuthApiWrapper = New AuthApiWrapper()
        Dim authCode As String = Await authApi.GetAccountAuthenticationToken()
        Dim luxSciResponse As LuxSciResponse = Nothing

        'build path and signature
        Dim path As String = accountRequestPath & accountId & "/users"
        Dim signature As String = AuthApiWrapper.GetHMACSignature(authCode & vbLf & "GET" & vbLf & path & vbLf & vbLf & vbLf)

        'send data to accountHttpClient
        Using accountHttpClient As AccountHttpClient = New AccountHttpClient()
            luxSciResponse = Await accountHttpClient.GetProfile(path, authCode, signature)
        End Using

        'return data, if success. else, throw exception
        If luxSciResponse.success = 1 Then
            Return luxSciResponse.data.ToString()
        Else
            Throw New ApiCallFailedException(luxSciResponse.error_message)
        End If
    End Function


    Public Async Function CreateAlias(ByVal [alias] As AliasRequest) As Task(Of Boolean)
        'Get authentication code
        Dim authApi As AuthApiWrapper = New AuthApiWrapper()
        Dim authCode As String = Await authApi.GetAccountAuthenticationToken()
        Dim luxSciResponse As LuxSciResponse = Nothing

        'build path and signature
        Dim path As String = accountRequestPath & accountId & "/aliases"
        Dim signature As String = AuthApiWrapper.GetHMACSignature(authCode & vbLf & "POST" & vbLf & path & vbLf & vbLf & AuthApiWrapper.ComputeSha256Hash(JsonConvert.SerializeObject([alias])) & vbLf)

        'call httpclient
        Using accountHttpClient As AccountHttpClient = New AccountHttpClient()
            luxSciResponse = Await accountHttpClient.CreateAlias(path, authCode, signature, [alias])
        End Using

        'throw error, if the success is not 1
        If luxSciResponse.success <> 1 Then
            Throw New ApiCallFailedException(luxSciResponse.error_message)
        Else
            Return True
        End If
    End Function

    Public Async Function GetMailSentReport(ByVal emailType As EmailType) As Task(Of String)
        'Get Authentication code
        Dim authApi As AuthApiWrapper = New AuthApiWrapper()
        Dim authCode As String = Await authApi.GetAccountAuthenticationToken()

        'build path And signature
        Dim luxSciResponse As LuxSciResponse = Nothing
        Dim path As String = String.Empty

        If emailType = EmailType.SMTP Then
            path = accountRequestPath & accountId & "/report/email/smtp/sent"
        Else
            path = accountRequestPath & accountId & "/report/email/webmail/sent"
        End If

        Dim signature As String = AuthApiWrapper.GetHMACSignature(authCode & vbLf & "GET" & vbLf & path & vbLf & vbLf & vbLf)

        'call httpclient
        Using accountHttpClient As AccountHttpClient = New AccountHttpClient()
            luxSciResponse = Await accountHttpClient.GetEmailSentReport(path, authCode, signature)
        End Using

        'throw exception, if success is not 1
        If luxSciResponse.success <> 1 Then
            Throw New ApiCallFailedException(luxSciResponse.error_message)
        Else
            Return luxSciResponse.data.ToString()
        End If
    End Function
End Class
