﻿Imports System.Configuration
Imports System.Net
Imports System.Net.Http

'LuxApiHttpClient is a base class which has all required common functions.
Public Class LuxApiHttpClient
    Implements IDisposable

    'members
    Protected client As HttpClient = Nothing
    Protected hostName As String = String.Empty
    Protected requestPath As String = String.Empty

    Public Sub New()
        'initialize host
        hostName = ConfigurationManager.AppSettings("Host")
    End Sub

    'Get HttpClient
    Protected Function GetHTTPClient(ByVal clientHandler As HttpClientHandler) As HttpClient
        If client Is Nothing Then
            client = New HttpClient(clientHandler) With {
                .BaseAddress = New Uri($"https://{hostName}")
            }
            client.DefaultRequestHeaders.Add("X-Version", "1")
            client.DefaultRequestHeaders.Accept.Clear()
            client.DefaultRequestHeaders.Accept.Add(New System.Net.Http.Headers.MediaTypeWithQualityHeaderValue("application/json"))
        End If

        Return client
    End Function

    'Get HttpClientHandler
    Protected Function GetHttpClientHandler(ByVal authenticationCode As String, ByVal hmacSignature As String) As HttpClientHandler
        Dim httpClientHandler As HttpClientHandler = New HttpClientHandler() With {
            .AllowAutoRedirect = True,
            .UseCookies = True,
            .CookieContainer = New CookieContainer()
        }
        Dim clientCookie As Cookie = New Cookie("signature", authenticationCode & ":" & hmacSignature)
        clientCookie.Expires = DateTime.Now.AddDays(2)
        clientCookie.Domain = hostName
        clientCookie.Path = "/"
        httpClientHandler.CookieContainer.Add(clientCookie)
        Return httpClientHandler
    End Function

    'Post call
    Public Async Function PostAsJsonAsync(ByVal obj As Object) As Task(Of LuxSciResponse)
        Dim response = Await client.PostAsJsonAsync(requestPath, obj)
        Return Await response.Content.ReadAsAsync(Of LuxSciResponse)()
    End Function

    'Get call
    Public Async Function GetDataAsync() As Task(Of LuxSciResponse)
        Dim response = Await client.GetAsync(requestPath)
        Return Await response.Content.ReadAsAsync(Of LuxSciResponse)()
    End Function

    'Send call
    Public Async Function SendAsync(ByVal request As HttpRequestMessage) As Task(Of LuxSciResponse)
        Dim response = Await client.SendAsync(request, HttpCompletionOption.ResponseContentRead)
        Return Await response.Content.ReadAsAsync(Of LuxSciResponse)()
    End Function

    Public Sub Dispose() Implements IDisposable.Dispose
        client.Dispose()
    End Sub
End Class
